vec3 GetEnviHUDSettings(bool getLapsSetting) {
string openEnvi = GetRootMap().MapInfo.CollectionName;
#if MP4 || TURBO
    if (openEnvi == "Canyon") return getLapsSetting ? Setting_CanyonHUDLaps : Setting_CanyonHUDSplits;
    else if (openEnvi == "Valley") return getLapsSetting ? Setting_ValleyHUDLaps : Setting_ValleyHUDSplits;
    else if (openEnvi == "Lagoon") return getLapsSetting ? Setting_LagoonHUDLaps : Setting_LagoonHUDSplits;
    else return getLapsSetting ? Setting_StadiumHUDLaps : Setting_StadiumHUDSplits;
#elif TMNEXT
    return getLapsSetting ? Setting_Stadium2020HUDLaps : Setting_Stadium2020HUDSplits;
#endif
}



// Used to detect cam 3 to disable HUD as it can break the projection
bool IsCam3Enabled() {
    if (GetApp().Viewport.Cameras.Length < 2) return false;
#if MP4 || TURBO
    return GetApp().Viewport.Cameras[1].DofFocusZ == 10.000;
#elif TMNEXT
    return GetApp().Viewport.Cameras[1].Fov == 80.000 || GetApp().Viewport.Cameras[1].Fov == 60.000;
#endif
}



double GetObjectDistance() {
    auto camera = GetApp().Viewport.Cameras[1];
    auto cameraPos = vec3(camera.NextLocation.tx, camera.NextLocation.ty, camera.NextLocation.tz);
    auto calCamPos = GetPosition() + (vec3(camera.NextLocation.xx, camera.NextLocation.yx, camera.NextLocation.zx)) + 
                                     (vec3(camera.NextLocation.xy, camera.NextLocation.yy, camera.NextLocation.zy)) +
                                     (vec3(camera.NextLocation.xz, camera.NextLocation.yz, camera.NextLocation.zz));
    return GetDistanceFromCamera(cameraPos, calCamPos);
}



#if TURBO
CGameControlCameraFree@ GetFreeCam() {return cast<CGameControlCameraFree@>(GetApp().CurrentPlayground.GameTerminals[0].CameraSet.CamsMaster.ManagedCams[6]);}
#endif



bool IsCam7Enabled() {
#if MP4
    double distance = GetObjectDistance();
    return distance < 5.0 || distance > 13.0;
#elif TMNEXT
    double distance = GetObjectDistance();
    return distance < 4.5 || distance > 13.0;
#elif TURBO
    return GetApp().CurrentPlayground !is null && GetApp().CurrentPlayground.GameTerminals.Length != 0 && GetFreeCam().IsActive;
#endif
}



vec3 ProjectVerticeSpace(CSceneVehicleVisState@ visState, vec3 point) {
#if MP4 || TMNEXT
    return visState.Position + (visState.Left * point.x) + (visState.Up * point.y) + (visState.Dir * point.z);
#elif TURBO
    return GetPosition() + (GetRoll() * point.x) + (GetPitch() * point.y) + (GetYaw() * point.z);
#endif
}


double GetDistanceFromCamera(vec3 position, vec3 cameraPos) {
    return Math::Sqrt(Math::Pow(position.x - cameraPos.x, 2) + Math::Pow(position.y - cameraPos.y, 2) + Math::Pow(position.z - cameraPos.z, 2) * 1.0);
}



// Combines both Render methods and adds conditions to display
void RenderHoloHUD() {
    if (IsCam3Enabled() || !Setting_EnableHUD || GetApp().InputPort.InputsMode != 0) return;
#if MP4
    if (!Setting_EnableMultiplayer) return;
#endif

    if (GetApp().Viewport.Cameras.Length < 2) return;
    iso4 cameraLocation = GetApp().Viewport.Cameras[1].NextLocation;
    vec3 cameraPos = vec3(cameraLocation.tx, cameraLocation.ty, cameraLocation.tz);

    // Check if its a new lap
    if (GetLapValue() != currentLap) {
        currentLap = GetLapValue();
        if (currentLap != 0) {newLap = true; epochTimerLap = Time::Stamp + Setting_HUDDisplayTime;}
    }

    // Check if a CP is hit
    if (currentCPCount != currCheckpointTimes.Length) {
        currentCPCount = currCheckpointTimes.Length;
        if (currentCPCount != 0) {showLapTime = newLap; epochTimerSplits = Time::Stamp + Setting_HUDDisplayTime;}
    }
    
    // Timer to display the HUDs
    if (!Setting_EnableExHud) {
        if ((epochTimerLap > Time::Stamp || Setting_AlwaysDisLaps)) RenderLapsHover(cameraPos);
        if (epochTimerLap < Time::Stamp) newLap = false;
        if (epochTimerSplits > Time::Stamp) RenderSplitsHover(cameraPos);
    } else {
        RenderLapsHover(cameraPos, true);
        RenderSplitsHover(cameraPos, true);
    }
}



void DrawHUDBase(vec3 objectAnchor, vec3 enviSettings, CSceneVehicleVisState@ visState, vec3 cameraPos, float sizeMultiplier, bool isSplits = false) {
    vec3 shortenedCenter = ProjectVerticeSpace(visState, vec3((enviSettings.x >= 0 ? (enviSettings.x - 0.15) : (enviSettings.x + 0.15)), enviSettings.y - 0.10, enviSettings.z));
    vec3 shortenedCenterRisen = ProjectVerticeSpace(visState, vec3((enviSettings.x >= 0 ? (enviSettings.x - 0.15) : (enviSettings.x + 0.15)), enviSettings.y - 0.085, enviSettings.z));
    vec2 firstTextAnchor = Camera::ToScreenSpace(objectAnchor);
    vec2 carCenter = Camera::ToScreenSpace(shortenedCenter);
    vec2 carCenterRisen = Camera::ToScreenSpace(shortenedCenterRisen);
    double scalableSize = (isSplits ? 0.545 : 0.525) * sizeMultiplier;

    if (Setting_Render3D) DrawSquare(enviSettings, vec3(enviSettings.x < 0 ? -scalableSize :  scalableSize, -0.015, 0), Setting_LineColour);
    else {
        float lengthScale = ((isSplits ? 355.0 : 340.0) * (Setting_FontSizeHUDScalable / 80.0)) / GetDistanceFromCamera(objectAnchor, cameraPos);
        float lengthNorm = (isSplits ? 81.0 : 77.5) * (Setting_FontSizeHUD / 18.0);

        nvg::BeginPath();
        if (Setting_AlwaysScaleHUD || !Setting_AlwaysScaleHUD && IsCam7Enabled()) nvg::Rect(firstTextAnchor, vec2(enviSettings.x > 0 ? -lengthScale : lengthScale, (10 / GetDistanceFromCamera(objectAnchor, cameraPos))));
        else nvg::Rect(firstTextAnchor, vec2(enviSettings.x > 0 ? -lengthNorm : lengthNorm, 2));
        nvg::FillColor(vec4(Setting_LineColour.x, Setting_LineColour.y, Setting_LineColour.z, 1));
        nvg::Fill();
        nvg::ClosePath();
    }

    if (Setting_EnableLineCenter) {
        vec2 bottomRight = Camera::ToScreenSpace(ProjectVerticeSpace(visState, vec3(enviSettings.x, enviSettings.y - 0.015, enviSettings.z)));
        nvg::BeginPath();
        DrawTriangle(carCenter, firstTextAnchor, bottomRight, Setting_LineColour, 1);
        DrawTriangle(carCenter, carCenterRisen, firstTextAnchor, Setting_LineColour, 1);
        nvg::Fill();
        nvg::ClosePath();
        
        nvg::BeginPath();
        nvg::FillColor(vec4(Setting_LineColour.x, Setting_LineColour.y, Setting_LineColour.z, 1));
        nvg::Circle(carCenter, (22.5 / GetDistanceFromCamera(shortenedCenter, cameraPos)));
        nvg::Fill();
        nvg::ClosePath();
    }
}



// Renders the lap count on the left of the car
void RenderLapsHover(vec3 cameraPos, bool showExample = false) {
    if (GetPlayer() is null || (!IsMapMultilap())) return;
    auto rootMap = GetRootMap();
    vec3 enviSettings = GetEnviHUDSettings(true);

#if MP4 || TURBO
    uint totalLaps = cast<CTrackManiaRaceNew@>(GetApp().CurrentPlayground).LapCount;
#elif TMNEXT
    uint totalLaps = rootMap.TMObjective_NbLaps;
#endif

#if MP4 || TMNEXT
    auto visState = VehicleState::GetVis(GetApp().GameScene, VehicleState::GetViewingPlayer()).AsyncState;
#elif TURBO
    auto visState = VehicleState::ViewingPlayerState();
#endif

    int lapCounter = GetLapValue() + 1;
    float sizeMultiplier = ((Setting_AlwaysScaleHUD || !Setting_AlwaysScaleHUD && IsCam7Enabled()) ? Setting_FontSizeHUDScalable / 80.0 : (Setting_FontSizeHUD / 18.0) * (0.2 * GetObjectDistance()));
    vec3 objectAnchorScreen = ProjectVerticeSpace(visState, enviSettings);
    vec3 objectAnchor = vec3(enviSettings.x,
                             enviSettings.y,
                             enviSettings.z);
    if (Camera::IsBehind(objectAnchorScreen)) return;

    vec2 lapAnchor = Camera::ToScreenSpace(objectAnchorScreen);
    nvg::FontSize((Setting_AlwaysScaleHUD || !Setting_AlwaysScaleHUD && IsCam7Enabled()) ? Setting_FontSizeHUDScalable / GetDistanceFromCamera(objectAnchorScreen, cameraPos) : Setting_FontSizeHUD);
    nvg::FontFace(fontFace);

    if (showExample) {
        TextString(objectAnchor, 'Laps: 1/3', visState, vec3(1, 1, 1), enviSettings, sizeMultiplier);
    } else {
        if (uint(lapCounter) <= totalLaps) TextString(objectAnchor, "Laps: " + lapCounter + "/" + totalLaps, visState, vec3(1, 1, 1), enviSettings, sizeMultiplier);
        else if (disableForOnlineMultilap) TextString(objectAnchor, "Laps: " + lapCounter + "/-", visState, vec3(1, 1, 1), enviSettings, sizeMultiplier);
        else TextString(objectAnchor, "Laps: " + totalLaps + "/" + totalLaps, visState, vec3(1, 1, 1), enviSettings, sizeMultiplier);
    }

    DrawHUDBase(objectAnchorScreen, enviSettings, visState, cameraPos, sizeMultiplier);
}



// Renders the run splits, lap time and CP time
void RenderSplitsHover(vec3 cameraPos, bool showExample = false) {
    if ((GetPlayer() is null || currentCheckpoint == MAX_INT || currentCheckpoint == 0) && !showExample) return;
    vec3 enviSettings = GetEnviHUDSettings(false);

#if MP4 || TMNEXT
    auto visState = VehicleState::GetVis(GetApp().GameScene, VehicleState::GetViewingPlayer()).AsyncState;
#elif TURBO
    auto visState = VehicleState::ViewingPlayerState();
#endif

    float sizeMultiplier = ((Setting_AlwaysScaleHUD || !Setting_AlwaysScaleHUD && IsCam7Enabled()) ? Setting_FontSizeHUDScalable / 80.0 : (Setting_FontSizeHUD / 18.0) * (0.2 * GetObjectDistance()));
    vec3 objectAnchorScreen = ProjectVerticeSpace(visState, enviSettings);
    vec3 objectAnchor = vec3(enviSettings.x,
                             enviSettings.y,
                             enviSettings.z);
    vec3 cpAnchorVec3 = vec3(enviSettings.x,
                             enviSettings.y + (0.125 * sizeMultiplier),
                             enviSettings.z);
    vec3 lapAnchorVec3 = vec3(enviSettings.x,
                              enviSettings.y + (0.250 * sizeMultiplier),
                              enviSettings.z);

    if (Camera::IsBehind(objectAnchorScreen)) return;
    nvg::FontSize((Setting_AlwaysScaleHUD || !Setting_AlwaysScaleHUD && IsCam7Enabled()) ? Setting_FontSizeHUDScalable / GetDistanceFromCamera(objectAnchorScreen, cameraPos) : Setting_FontSizeHUD);
    nvg::FontFace(fontFace);

    if (showExample) {
        TextString(objectAnchor, "+" + formatSplit(165), visState, Setting_NegativeDelta, enviSettings, sizeMultiplier);
        TextString(cpAnchorVec3, formatTime(65433) + '', visState, vec3(1, 1, 1), enviSettings, sizeMultiplier);
        TextString(lapAnchorVec3, formatTime(27454) + '', visState, Setting_LapCountColour, enviSettings, sizeMultiplier);
    } else {
#if MP4 || TURBO
        CTrackManiaRaceInterface@ interfaceTM = cast<CTrackManiaRaceInterface@>(GetApp().CurrentPlayground.Interface);
        string position = interfaceTM.CheckPointInfo_CurrentRacePositionText0;
        bool condition = !isPlayingOnline && showLapTime;
#elif TMNEXT
        string position = GetRoundPosition();
        bool condition = GetRootMap().TMObjective_IsLapRace && showLapTime && !IsModeRounds();
#endif
        if (currentCheckpoint != MAX_INT && replayLoaded) {
            TextString(cpAnchorVec3, formatTime(currentCheckpoint) + '', visState, vec3(1, 1, 1), enviSettings, sizeMultiplier);

            if (condition) TextString(lapAnchorVec3, formatTime(lapTime2Display) + '', visState, Setting_LapCountColour, enviSettings, sizeMultiplier);
            else if (isPlayingOnline) {
#if MP4
                if (IsModeRounds()) TextString(lapAnchorVec3, GetRoundPosition(), visState, Setting_LapCountColour, enviSettings, sizeMultiplier);
                else TextString(lapAnchorVec3, position.Replace(")", ""), visState, Setting_LapCountColour, enviSettings, sizeMultiplier);
#elif TURBO
                TextString(lapAnchorVec3, position.Replace(")", ""), visState, Setting_LapCountColour, enviSettings, sizeMultiplier);
#elif TMNEXT
                TextString(lapAnchorVec3, GetRoundPosition(), visState, Setting_LapCountColour, enviSettings, sizeMultiplier);
#endif
            }
        } else if (currentCheckpoint != MAX_INT) {
            TextString(objectAnchor, formatTime(currentCheckpoint) + '', visState, vec3(1, 1, 1), enviSettings, sizeMultiplier);

            if (!isPlayingOnline && showLapTime) TextString(cpAnchorVec3, formatTime(lapTime2Display) + '', visState, Setting_LapCountColour, enviSettings, sizeMultiplier);
            else if (isPlayingOnline) {
#if MP4 || TMNEXT
                if (IsModeRounds()) {
                    TextString(cpAnchorVec3, GetRoundPosition(), visState, Setting_LapCountColour, enviSettings, sizeMultiplier);
                }
                else {
                    TextString(cpAnchorVec3, position.Replace(")", ""), visState, Setting_LapCountColour, enviSettings, sizeMultiplier);
                }
#elif TURBO
                TextString(cpAnchorVec3, position.Replace(")", ""), visState, Setting_LapCountColour, enviSettings, sizeMultiplier);
#endif
            }
        }

        if (replayLoaded) {
            if (currentCPSplit > 0) {
                TextString(objectAnchor, "+" + formatSplit(Math::Abs(currentCPSplit)), visState, Setting_NegativeDelta, enviSettings, sizeMultiplier);
            }
            else if (currentCPSplit < 0) {
                TextString(objectAnchor, "-" + formatSplit(Math::Abs(currentCPSplit)), visState, Setting_PositiveDelta, enviSettings, sizeMultiplier);
            }
            else {
                TextString(objectAnchor, "+0:00.000", visState, Setting_EqualDelta, enviSettings, sizeMultiplier);
            }
        }
    }
    DrawHUDBase(objectAnchorScreen, enviSettings, visState, cameraPos, sizeMultiplier, true);
}



void DrawTriangle(vec2 pos1, vec2 pos2, vec2 pos3, vec3 colour, float alpha = 0.5, bool outline = false) {
    nvg::BeginPath();
    nvg::MoveTo(pos1);
    nvg::LineTo(pos2);
    nvg::LineTo(pos3);
    nvg::FillColor(vec4(colour.x, colour.y, colour.z, alpha));
    nvg::Fill();
    nvg::ClosePath();
}



void DrawSquare(vec3 coords, vec3 size, vec3 colour) {
#if MP4 || TMNEXT
    auto visState = VehicleState::GetVis(GetApp().GameScene, VehicleState::GetViewingPlayer()).AsyncState;
#elif TURBO
    auto visState = VehicleState::ViewingPlayerState();
#endif

    vec2 topLeft = Camera::ToScreenSpace(ProjectVerticeSpace(visState, coords));
    vec2 bottomRight = Camera::ToScreenSpace(ProjectVerticeSpace(visState, coords + size));

    DrawTriangle(topLeft, Camera::ToScreenSpace(ProjectVerticeSpace(visState, coords + vec3(size.x, 0, 0))), bottomRight, colour, 1);
    DrawTriangle(bottomRight, Camera::ToScreenSpace(ProjectVerticeSpace(visState, coords + vec3(0, size.y, 0))), topLeft, colour, 1);
}



void TextString(vec3 position, const string &in text, CSceneVehicleVisState@ visState, vec3 colour, vec3 enviSettings, float sizeMultiplier) {
    nvg::FillColor(vec4(colour, 1));
    nvg::TextAlign((position.x >= 0 ? nvg::Align::Right : nvg::Align::Left) | nvg::Align::Bottom);
    vec2 position1 = Camera::ToScreenSpace(ProjectVerticeSpace(visState, position));

    if (Setting_Render3D) {
        float positionX = (enviSettings.x >= 0) ? position.x + (0.065 * text.Length) : position.x - (0.065 * text.Length);
        vec2 difference = (position1 - Camera::ToScreenSpace(ProjectVerticeSpace(visState, vec3(positionX, position.y, position.z)))) / text.Length;

        vec2 nextPosition = position1;
        vec2 spacing = vec2(0);
        for (int i = ((enviSettings.x < 0) ? 0 : text.Length - 1); ((enviSettings.x < 0) ? i < text.Length : i >= 0); ((enviSettings.x < 0) ? i++ : i--)) {
            string char = text.SubStr(i, 1);
            nvg::Text(nextPosition, char);
            
            float scaler = ((char == ':' || char == '.' || char == '/' || char == ' ') ? 0.75 : 1) * sizeMultiplier;

            if (position.x >= 0) {
                spacing += (difference * scaler);
                nextPosition = position1 - spacing;
            }
            else {
                spacing -= (difference * scaler);
                nextPosition = position1 + spacing;
            }
        }
    } else {nvg::Text(position1, text);}
}



bool IsModeRounds() {
#if MP4 || TURBO
    return GetManiaplanetAPI().CurrentServerModeName == 'Rounds' || GetManiaplanetAPI().CurrentServerModeName == 'Chase' || GetManiaplanetAPI().CurrentServerModeName == 'Cup' ||
           GetManiaplanetAPI().CurrentServerModeName == 'Laps' || GetManiaplanetAPI().CurrentServerModeName == 'Team' || GetManiaplanetAPI().CurrentServerModeName == 'RoundsBase';
#elif TMNEXT
    return GetManiaplanetAPI().CurrentServerModeName == 'TM_Rounds_Online' || GetManiaplanetAPI().CurrentServerModeName == 'TM_Teams_Online' || GetManiaplanetAPI().CurrentServerModeName == 'TM_Laps_Online' ||
           GetManiaplanetAPI().CurrentServerModeName == 'TM_Cup_Online' || GetManiaplanetAPI().CurrentServerModeName == 'TM_Knockout_Online';
#endif
}