namespace UI
{
    bool RedButton(const string &in text) { return UI::ButtonColored(text, 0.0f, 0.6f, 0.6f); }
    bool RedButton(const string &in text, vec2 size) { return UI::ButtonColored(text, 0.0f, 0.75f, 0.75f, size); }
    bool GreenButton(const string &in text) { return UI::ButtonColored(text, 0.33f, 0.6f, 0.6f); }
    bool OrangeButton(const string &in text) { return UI::ButtonColored(text, 0.1f, 0.6f, 0.6f); }
    bool GreyButton(const string &in text) { return UI::ButtonColored(text, 0.3f, 0.0f, 0.3f); }
    bool GreyButton(const string &in text, vec2 size) { return UI::ButtonColored(text, 0.55f, 0.55f, 0.55f, size); }
}

void LeftPadding(int width, const string &in text) {
    if (width != 0) {
        UI::Dummy(vec2(width, 0));
        UI::SameLine();
    }
    UI::Text(text);
}

void LeftPaddingDummy(int width) {
    if (width != 0) {
        UI::Dummy(vec2(width, 0));
        UI::SameLine();
    }
}

// Runs the UI and associated functionality
void runGUI() {
    UI::SetNextWindowPos(int(Setting_UIPos.x), int(Setting_UIPos.y), Setting_LockUIPos ? UI::Cond::Always : UI::Cond::FirstUseEver);
    int window_flags = UI::WindowFlags::NoTitleBar | UI::WindowFlags::NoCollapse | UI::WindowFlags::AlwaysAutoResize | UI::WindowFlags::NoDocking;

    auto rootMap = GetRootMap();
    disableForOnlineMultilap = !(((isPlayingOnline && !modeIsTimeAttack && !rootMap.TMObjective_IsLapRace || isPlayingOnline && !modeIsTimeAttack && rootMap.TMObjective_IsLapRace) || (isPlayingOnline && modeIsTimeAttack && !rootMap.TMObjective_IsLapRace)) || !isPlayingOnline);
    
    string nameText = '';
    if (loadedGhost.nickname != '') { nameText = formatTime(loadedGhost.raceTime) + " \\$zby \\$s" + openplanetUIDisplay(loadedGhost.nickname); replayLoaded = true;}
    else {nameText = 'Finding Replay...'; replayLoaded = false;}

    if (!Setting_Enable || Setting_DisableGUIforHUD) {return;}
#if MP4
    if (isPlayingOnline && !Setting_EnableMultiplayer) {return;}
#endif

#if MP4 || TURBO
    uint totalLaps = GetApp().CurrentPlayground is null ? rootMap.TMObjective_NbLaps : cast<CTrackManiaRaceNew@>(GetApp().CurrentPlayground).LapCount;
#elif TMNEXT
    uint totalLaps = rootMap.TMObjective_NbLaps;
#endif

    UI::PushFont(font);
    UI::Begin("Test", window_flags);
        UI::BeginGroup();
        // Versing Replay Information
        UI::Separator();
        UI::BeginTable("Start", 3, UI::TableFlags::SizingFixedFit);
            UI::TableNextRow();
            UI::TableNextColumn();
            LeftPadding(Setting_PaddingWidth, "\\$s" + Icons::UserCircleO);
            UI::TableNextColumn();
            LeftPadding(Setting_PaddingWidth, nameText);
            UI::TableNextColumn();
            if (Setting_QuickAccess) {
                if (Setting_PaddingWidth != 0) {
                    UI::Dummy(vec2(Setting_PaddingWidth, 0));
                    UI::SameLine();
                }
                if (UI::GreyButton("\\$fff" + Icons::Cogs, vec2(Setting_FontSize, Setting_FontSize))) {
                    replayListAll = !replayListAll;
                }
            }
        UI::EndTable();
        UI::Separator();
        
        // General Stats
        if (!disableForOnlineMultilap) {
            UI::BeginTable("General Stats", 4, UI::TableFlags::SizingFixedFit);
                // Current Checkpoint
                UI::TableNextRow();
                UI::TableNextColumn();
                LeftPadding(Setting_PaddingWidth, "\\$s\\$ddd" + Icons::VideoCamera);
                UI::TableNextColumn();
                LeftPadding(Setting_PaddingWidth, "\\$sCurrent CP");
                UI::TableNextColumn();
                if (GetCPTime() != MAX_INT && GetCPTime() != 0) {
                    LeftPadding(Setting_PaddingWidth, "\\$bbb" + formatTime(GetCPTime()));
                    if (replayLoaded) {
                        UI::TableNextColumn();
                        RenderSplit(currentCPSplit);
                    }  
                }
                else {LeftPadding(Setting_PaddingWidth, "\\$bbb" + "00:00.000");}

                // Previous Time
                UI::TableNextRow();
                UI::TableNextColumn();
                LeftPadding(Setting_PaddingWidth, "\\$ddd" + Icons::ClockO);
                UI::TableNextColumn();
                LeftPadding(Setting_PaddingWidth, "Previous Run");
                UI::TableNextColumn();
                if (runTime2Display != 0) {
                    LeftPadding(Setting_PaddingWidth, "\\$bbb" + formatTime(runTime2Display));
                    if (replayLoaded) {
                        UI::TableNextColumn();
                        RenderSplit(previousRunSplit);
                    }
                }
                else {LeftPadding(Setting_PaddingWidth, "\\$bbb" + "00:00.000");}

                // Best Time
                UI::TableNextRow();
                UI::TableNextColumn();
                LeftPadding(Setting_PaddingWidth, "\\$ddd" + Icons::ClockO);
                UI::TableNextColumn();
                LeftPadding(Setting_PaddingWidth, "Fastest Run");
                UI::TableNextColumn();
                if (fastestRun2Display != 0) {
                    LeftPadding(Setting_PaddingWidth, "\\$bbb" + formatTime(fastestRun2Display));
                    if (replayLoaded) {
                        UI::TableNextColumn();
                        RenderSplit(bestRunSplit);
                    }
                }
                else {LeftPadding(Setting_PaddingWidth, "\\$bbb" + "00:00.000");}

                // Lap count for lap stats disabled
                if (IsMapMultilap() && !Setting_EnableLapStats) {
                    UI::TableNextRow();
                    UI::TableNextColumn();
                    LeftPadding(Setting_PaddingWidth, "\\$ddd" + Icons::FlagCheckered);
                    UI::TableNextColumn();
                    LeftPadding(Setting_PaddingWidth, "Lap");
                    UI::TableNextColumn();
                    if (uint(GetLapValue() + 1) <= totalLaps) {LeftPadding(Setting_PaddingWidth, "\\$bbb" + (GetLapValue() + 1) + " \\$fff/ \\$bbb" + totalLaps);}
                    else {LeftPadding(Setting_PaddingWidth, "\\$bbb" + totalLaps + " \\$fff/ \\$bbb" + totalLaps);}
                }
            UI::EndTable();
        }
        // Multilap Stats
        if (IsMapMultilap() && (Setting_EnableLapStats || disableForOnlineMultilap)) {
            if (!disableForOnlineMultilap) {UI::Separator();}
            UI::BeginTable("Multilap Stats", 4, UI::TableFlags::SizingFixedFit);
                // Lap count
                if (!disableForOnlineMultilap) {
                    UI::TableNextRow();
                    UI::TableNextColumn();
                    LeftPadding(Setting_PaddingWidth, "\\$ddd" + Icons::FlagCheckered);
                    UI::TableNextColumn();
                    LeftPadding(Setting_PaddingWidth, "Lap");
                    UI::TableNextColumn();
                    if (uint(GetLapValue() + 1) <= totalLaps) {LeftPadding(Setting_PaddingWidth, "\\$bbb" + (GetLapValue() + 1) + " \\$fff/ \\$bbb" + totalLaps);}
                    else {LeftPadding(Setting_PaddingWidth, "\\$bbb" + totalLaps + " \\$fff/ \\$bbb" + totalLaps);}
                } else {
                    // Current Checkpoint
                    UI::TableNextRow();
                    UI::TableNextColumn();
                    LeftPadding(Setting_PaddingWidth, "\\$s\\$ddd" + Icons::VideoCamera);
                    UI::TableNextColumn();
                    LeftPadding(Setting_PaddingWidth, "\\$sCurrent CP");
                    UI::TableNextColumn();
                    if (GetCPTime() != MAX_INT) {
                        LeftPadding(Setting_PaddingWidth, "\\$bbb" + formatTime(GetCPTime()));
                        if (replayLoaded) {
                            UI::TableNextColumn();
                            RenderSplit(currentCPSplit);
                        }  
                    }
                    else {LeftPadding(Setting_PaddingWidth, "\\$bbb" + "00:00.000");}
                }

                // Previous Lap
                UI::TableNextRow();
                UI::TableNextColumn();
                LeftPadding(Setting_PaddingWidth, "\\$bbb" + "\\$ddd" + Icons::ClockO);
                UI::TableNextColumn();
                LeftPadding(Setting_PaddingWidth, "Previous Lap");
                UI::TableNextColumn();
                if (lapTime2Display != 0 && lapTime2Display != MAX_INT) {
                    LeftPadding(Setting_PaddingWidth, "\\$bbb" + formatTime(lapTime2Display));
                    UI::TableNextColumn();
                    RenderSplit(previousLapSplit);
                }
                else {LeftPadding(Setting_PaddingWidth, "\\$bbb" + "00:00.000");}

                // Fastest Lap
                UI::TableNextRow();
                UI::TableNextColumn();
                LeftPadding(Setting_PaddingWidth, "\\$ddd" + Icons::ClockO);
                UI::TableNextColumn();
                LeftPadding(Setting_PaddingWidth, "Fastest Lap");
                UI::TableNextColumn();
                if (fastestLap2Display != 0 && fastestLap2Display != MAX_INT) {LeftPadding(Setting_PaddingWidth, "\\$bbb" + formatTime(fastestLap2Display));}
                else {LeftPadding(Setting_PaddingWidth, "\\$bbb" + "00:00.000");}
            UI::EndTable();
        }
        UI::EndGroup();
    if (!Setting_LockUIPos) Setting_UIPos = UI::GetWindowPos();
    UI::End();
    UI::PopFont();
}

#if MP4 || TURBO
void GenerateReplayCategories() {
    auto dataFileMgr = GetDataMgr();
    if (dataFileMgr.Ghosts.Length != 0) {
        indexToDisplayDownloaded = {};
        indexToDisplayMedals = {};
        indexToDisplayTotal = {};
        int pbIndex = GetPBIndex();
        if (pbIndex != -1) {
            indexToDisplayTotal.InsertLast(pbIndex);
            indexToDisplayDownloaded.InsertLast(pbIndex);
        }

        for (uint i = 0; i < dataFileMgr.Ghosts.Length; i++) {
            if (GhostResult(i) is null) {continue;}
            if (GhostResult(i).Time == -1) {continue;}
            bool addToList = true;
            for (uint k = 0; k < indexToDisplayTotal.Length; k++) {
                if (GhostResult(i).Time == GhostResult(indexToDisplayTotal[k]).Time) {
                    uint similarCPCount = 0;
                    for (uint l = 0; l < GhostResult(i).Checkpoints.Length; l++) {
                        if (GhostResult(i).Checkpoints[l] == GhostResult(indexToDisplayTotal[k]).Checkpoints[l]) {similarCPCount++;}
                    }
                    if (similarCPCount == GhostResult(i).Checkpoints.Length) {addToList = false;}
                }
            }
            if (addToList) {
                indexToDisplayTotal.InsertLast(i);
                if (!NameIsNotMedalOrPB(i)) {indexToDisplayMedals.InsertLast(i);}
                else if (dataFileMgr.Ghosts[i].Nickname != GetPlayerName()) {indexToDisplayDownloaded.InsertLast(i);}     
            }
        }
    }
}

#elif TMNEXT
void GenerateReplayCategories() {
    auto dataFileMgr = GetDataMgr();
    if (GetApp().PlaygroundScript !is null) {
        indexToDisplayDownloaded = {};
        if (dataFileMgr.Ghosts.Length != 0) {
            for (uint i = 0; i < dataFileMgr.Ghosts.Length; i++) {
                if (dataFileMgr.Ghosts[i].Result is null) continue;
                if (dataFileMgr.Ghosts[i].Result.Time == MAX_INT) continue;
                if (dataFileMgr.Ghosts[i].Nickname.Contains('Personal best')) indexToDisplayDownloaded.InsertAt(0, i);
            }
        }
        if (GetApp().Network.ClientManiaAppPlayground !is null) {
            auto onlineGhosts = GetApp().Network.ClientManiaAppPlayground.DataFileMgr.Ghosts;
            if (onlineGhosts.Length != 0) {
                for (uint i = 0; i < onlineGhosts.Length; i++) {
                    if (onlineGhosts[i].Result is null) continue;
                    if (onlineGhosts[i].Result.Time == MAX_INT) continue;
                    indexToDisplayDownloaded.InsertLast(i);
                }
            }
        }
    } else {
        if (dataFileMgr.Ghosts.Length != 0) {
            indexToDisplayDownloaded = {};
            for (int i = dataFileMgr.Ghosts.Length - 1; i >= ghostCountStartTM2020; i--) {
                if (dataFileMgr.Ghosts[i].Result is null) continue;
                if (dataFileMgr.Ghosts[i].Result.Time == MAX_INT) continue;
                if (dataFileMgr.Ghosts[i].Nickname.Contains('Personal best')) indexToDisplayDownloaded.InsertAt(0, i);
                else if (!dataFileMgr.Ghosts[i].Nickname.Contains(GetPlayerName())) indexToDisplayDownloaded.InsertLast(i); 
            }
        }
    }
}           
#endif

void runGUIReplayList() {
    if (!replayListAll) {
        replayListAll = false;
		return;
	}

    auto rootMap = GetRootMap();
#if MP4
    bool condition = !isPlayingOnline;
    CGameDataFileManagerScript@ dataFileMgr;
    if (GetApp().PlaygroundScript !is null) @dataFileMgr = GetDataMgr();
#elif TURBO
    bool condition = !isPlayingOnline;
    CGameDataManagerScript@ dataFileMgr;
    if (GetApp().PlaygroundScript !is null) @dataFileMgr = GetDataMgr();
#elif TMNEXT
    bool condition = true;
    auto dataFileMgr = GetDataMgr();
    GenerateReplayCategories();
#endif

    UI::PushFont(font);
    UI::SetNextWindowSize(Setting_SizeX * (Setting_FontSize / 16), Setting_SizeY * (Setting_FontSize / 16), UI::Cond::Always);
    if (UI::Begin("\\$08f" + Icons::Refresh + "\\$z Change Replay", replayListAll, UI::WindowFlags::HorizontalScrollbar | UI::WindowFlags::NoCollapse)) {
        UI::BeginChild('ScrollingRegion', vec2(0, -(UI::GetFrameHeightWithSpacing() + 10)), false, UI::WindowFlags::HorizontalScrollbar);
        int counter = 1;
        
        UI::BeginTable("PrevRuns", 2, UI::TableFlags::SizingFixedFit);
            UI::TableNextRow();
            UI::TableNextColumn();
            LeftPadding(Setting_PaddingWidth, Icons::Flag);
            UI::TableNextColumn();
            LeftPadding(Setting_PaddingWidth, "Previous Runs");
            UI::TableNextColumn();
        UI::EndTable();
        UI::Separator();
        UI::BeginTable("User Runs", 4, UI::TableFlags::SizingFixedFit);
        UI::TableNextRow();
        UI::TableNextColumn();
        for (uint i = 0; i < previousRunTimes.Length; i++) {
            LeftPaddingDummy(Setting_PaddingWidth);
            UI::PushID('Use' + counter);
            if (UI::GreenButton("Use")) {
                selectedIndex = i;
                userSelectedReplay = true;
                selectedDediIndex = -1;
                selectedSTMIndex = -1;
                loadedGhost = previousRunTimes[i];
                if (Setting_EnableNotif) {
                    UI::ShowNotification("\\$s\\$fff" + Icons::InfoCircle + " Replay Successfully Changed!",
                                "\\$s\\$fff" + Icons::ClockO + " Time: " + formatTime(loadedGhost.raceTime) + "\n" + Icons::UserCircleO + " Name: " + openplanetUIDisplay(loadedGhost.nickname),
                                vec4(Setting_NotificationColour.x, Setting_NotificationColour.y, Setting_NotificationColour.z, 0), 1500);
                }                    
            }
            UI::PopID();

            UI::TableNextColumn();
            LeftPadding(Setting_PaddingWidth, "\\$s" + formatTime(previousRunTimes[i].raceTime));
            UI::TableNextColumn();
            LeftPadding(Setting_PaddingWidth, "|");
            UI::TableNextColumn();
            LeftPadding(Setting_PaddingWidth, openplanetUIDisplay(previousRunTimes[i].nickname));
            UI::TableNextRow();
            UI::TableNextColumn();
            counter++;
        }
        if (counter == 1) {LeftPadding(Setting_PaddingWidth, "No Replays Found. Complete A Run.");}
        UI::EndTable();
        UI::Separator();
        UI::Text("");

        if (Setting_IncDownloadedTimes || isPlayingOnline) {
            UI::BeginTable("DownloadedRuns", 2, UI::TableFlags::SizingFixedFit);
                UI::TableNextRow();
                UI::TableNextColumn();
                LeftPadding(Setting_PaddingWidth, Icons::Download);
                UI::TableNextColumn();
                LeftPadding(Setting_PaddingWidth, "Downloaded Replays");
                UI::TableNextColumn();
            UI::EndTable();
            UI::Separator();
            UI::BeginTable("Downloaded Runs", 4, UI::TableFlags::SizingFixedFit);
                UI::TableNextRow();
                UI::TableNextColumn();
#if MP4
                if (Setting_DediSoloEnable || isPlayingOnline) {
                    for (uint i = 0; i < dedimaniaLogins.Length; i++) {
                        if (dediReplayCPs[i].Length == 0) continue;
                        LeftPaddingDummy(Setting_PaddingWidth);
                        UI::PushID('Use' + counter);
                        if (UI::GreenButton("Use")) {
                            selectedDediIndex = i;
                            selectedIndex = -1;
                            selectedSTMIndex = -1;
                            userSelectedReplay = true;
                            loadedGhost = CreateGhost(selectedDediIndex);
                            if (Setting_EnableNotif) {
                                UI::ShowNotification("\\$s\\$fff" + Icons::InfoCircle + " Replay Successfully Changed!",
                                            "\\$s\\$fff" + Icons::ClockO + " Time: " + formatTime(loadedGhost.raceTime) + "\n" + Icons::UserCircleO + " Name: " + openplanetUIDisplay(loadedGhost.nickname),
                                            vec4(Setting_NotificationColour.x, Setting_NotificationColour.y, Setting_NotificationColour.z, 0), 1500);
                            }
                        }
                        UI::PopID();

                        UI::TableNextColumn();
                        LeftPadding(Setting_PaddingWidth, "\\$s" + formatTime(dediReplayCPs[i][dediReplayCPs[i].Length - 1]));
                        UI::TableNextColumn();
                        LeftPadding(Setting_PaddingWidth, "|");
                        UI::TableNextColumn();
                        if (i == 0 || i == 1) LeftPadding(Setting_PaddingWidth, openplanetUIDisplay(GetApp().CurrentProfile.AccountSettings.NickName));
                        else LeftPadding(Setting_PaddingWidth, openplanetUIDisplay(dedimaniaLogins[i]));
                        UI::TableNextRow();
                        UI::TableNextColumn();
                        counter++;
                    }
                }
#endif
                if (condition) {
                    for (uint i = 0; i < indexToDisplayDownloaded.Length; i++) {
                        LeftPaddingDummy(Setting_PaddingWidth);
                        UI::PushID('Use' + counter);
                        if (UI::GreenButton("Use")) {
                            selectedIndex = indexToDisplayDownloaded[i];
                            userSelectedReplay = true;
#if MP4 || TURBO
                            selectedDediIndex = -1;
                            selectedSTMIndex = -1;
                            loadedGhost = CreateGhost(dataFileMgr.Ghosts[indexToDisplayDownloaded[i]]);
                            if (Setting_EnableNotif) {
                                UI::ShowNotification("\\$s\\$fff" + Icons::InfoCircle + " Replay Successfully Changed!",
                                            "\\$s\\$fff" + Icons::ClockO + " Time: " + formatTime(loadedGhost.raceTime) + "\n" + Icons::UserCircleO + " Name: " + openplanetUIDisplay(loadedGhost.nickname),
                                            vec4(Setting_NotificationColour.x, Setting_NotificationColour.y, Setting_NotificationColour.z, 0), 1500);
                            }
#elif TMNEXT
                            if (i == 0 && dataFileMgr.Ghosts.Length != 0 && uint(indexToDisplayDownloaded[i]) <= dataFileMgr.Ghosts.Length && 
                               (dataFileMgr.Ghosts[indexToDisplayDownloaded[i]].Nickname.Contains('Personal best') && dataFileMgr.Ghosts[indexToDisplayDownloaded[i]].Result.Time != MAX_INT)) {
                                loadedGhost = CreateGhost(dataFileMgr.Ghosts[indexToDisplayDownloaded[i]]);
                            } else loadedGhost = CreateGhost(GetApp().Network.ClientManiaAppPlayground.DataFileMgr.Ghosts[indexToDisplayDownloaded[i]]);
                            if (Setting_EnableNotif) {
                                UI::ShowNotification("\\$s\\$fff" + Icons::InfoCircle + " Replay Successfully Changed!",
                                            "\\$s\\$fff" + Icons::ClockO + " Time: " + formatTime(loadedGhost.raceTime) + "\n" + Icons::UserCircleO + " Name: " + openplanetUIDisplay(loadedGhost.nickname),
                                            vec4(Setting_NotificationColour.x, Setting_NotificationColour.y, Setting_NotificationColour.z, 0), 1500);
                            }
#endif
                        }
                        UI::PopID();

#if MP4 || TURBO
                        if (GhostResult(indexToDisplayDownloaded[i]) is null) {continue;}
                        if (GhostResult(indexToDisplayDownloaded[i]).Time == -1) {continue;}
                        UI::TableNextColumn();
                        LeftPadding(Setting_PaddingWidth, formatTime(GhostResult(indexToDisplayDownloaded[i]).Time));
#elif TMNEXT
                        CGameGhostScript@ ghost;
                        if (i == 0 && dataFileMgr.Ghosts.Length != 0 && uint(indexToDisplayDownloaded[i]) <= dataFileMgr.Ghosts.Length && 
                           (dataFileMgr.Ghosts[indexToDisplayDownloaded[i]].Nickname.Contains('Personal best') && dataFileMgr.Ghosts[indexToDisplayDownloaded[i]].Result.Time != MAX_INT)) {
                            @ghost = dataFileMgr.Ghosts[indexToDisplayDownloaded[i]];
                        } else @ghost = GetApp().Network.ClientManiaAppPlayground.DataFileMgr.Ghosts[indexToDisplayDownloaded[i]];

                        if (ghost.Result is null) {continue;}
                        if ((ghost.Result.Time == MAX_INT)) {continue;}
                        UI::TableNextColumn();
                        LeftPadding(Setting_PaddingWidth, formatTime(ghost.Result.Time));
#endif
                        UI::TableNextColumn();
                        LeftPadding(Setting_PaddingWidth, "|");
                        UI::TableNextColumn();
#if MP4 || TURBO
                        LeftPadding(Setting_PaddingWidth, openplanetUIDisplay(dataFileMgr.Ghosts[indexToDisplayDownloaded[i]].Nickname));
#elif TMNEXT
                        LeftPadding(Setting_PaddingWidth, openplanetUIDisplay(ghost.Nickname));
#endif
                        UI::TableNextRow();
                        UI::TableNextColumn();
                        counter++;
                    }
                }
                if (indexToDisplayDownloaded.Length == 0 && dediReplayCPs[0].Length == 0 && dediReplayCPs[1].Length == 0 && dediReplayCPs[2].Length == 0) {LeftPadding(Setting_PaddingWidth, "No Replays Downloaded.");}
            UI::EndTable();
            UI::Separator();
        }

#if MP4 || TURBO
        if (Setting_IncMedalTimes && !isPlayingOnline) {
            UI::Text("");
            UI::BeginTable("MedalTimes", 2, UI::TableFlags::SizingFixedFit);
                UI::TableNextRow();
                UI::TableNextColumn();
                LeftPadding(Setting_PaddingWidth, Icons::Trophy);
                UI::TableNextColumn();
                LeftPadding(Setting_PaddingWidth, "Medal Times");
                UI::TableNextColumn();
            UI::EndTable();
            UI::Separator();
            UI::BeginTable("Medal Runs", 4, UI::TableFlags::SizingFixedFit);
            UI::TableNextRow();
            UI::TableNextColumn();
#endif
#if TURBO
            int mapSTMIndex = Text::ParseInt(rootMap.MapName) - 1;

            if (mapSTMIndex >= 0 && mapSTMIndex < 200) {
                STMRecord@ recordSTM = STMRecords[mapSTMIndex];
                LeftPaddingDummy(Setting_PaddingWidth);
                UI::PushID('Use' + counter);
                if (UI::GreenButton("Use")) {
                    selectedIndex = -1;
                    selectedDediIndex = -1;
                    selectedSTMIndex = mapSTMIndex;
                    userSelectedReplay = true;
                    loadedGhost = CreateGhost(STMRecords[mapSTMIndex]);
                    if (Setting_EnableNotif) {
                        UI::ShowNotification("\\$s\\$fff" + Icons::InfoCircle + " Replay Successfully Changed!",
                                    "\\$s\\$fff" + Icons::ClockO + " Time: " + formatTime(recordSTM.raceTime) + "\n" + Icons::UserCircleO + " Name: " + openplanetUIDisplay(recordSTM.playerName),
                                    vec4(Setting_NotificationColour.x, Setting_NotificationColour.y, Setting_NotificationColour.z, 0), 1500);
                    }
                }
                UI::PopID();

                UI::TableNextColumn();
                LeftPadding(Setting_PaddingWidth, "\\$s" + formatTime(recordSTM.raceTime));
                UI::TableNextColumn();
                LeftPadding(Setting_PaddingWidth, "|");
                UI::TableNextColumn();
                LeftPadding(Setting_PaddingWidth, openplanetUIDisplay(recordSTM.playerName));
                UI::TableNextRow();
                UI::TableNextColumn();
                counter++;
            }
#endif
#if MP4 || TURBO
            for (uint i = 0; i < indexToDisplayMedals.Length; i++) {
                LeftPaddingDummy(Setting_PaddingWidth);
                UI::PushID('Use' + counter);
                if (UI::GreenButton("Use")) {
                    selectedIndex = indexToDisplayMedals[i];
                    selectedDediIndex = -1;
                    selectedSTMIndex = -1;
                    userSelectedReplay = true;
                    loadedGhost = CreateGhost(dataFileMgr.Ghosts[indexToDisplayMedals[i]]);
                    if (Setting_EnableNotif) {
                        UI::ShowNotification("\\$s\\$fff" + Icons::InfoCircle + " Replay Successfully Changed!",
                                    "\\$s\\$fff" + Icons::ClockO + " Time: " + formatTime(GhostResult(indexToDisplayMedals[i]).Time) + "\n" + Icons::UserCircleO + " Name: " + openplanetUIDisplay(dataFileMgr.Ghosts[indexToDisplayMedals[i]].Nickname),
                                    vec4(Setting_NotificationColour.x, Setting_NotificationColour.y, Setting_NotificationColour.z, 0), 1500);
                    }
                }
                UI::PopID();

                UI::TableNextColumn();
                LeftPadding(Setting_PaddingWidth, "\\$s" + formatTime(GhostResult(indexToDisplayMedals[i]).Time));
                UI::TableNextColumn();
                LeftPadding(Setting_PaddingWidth, "|");
                UI::TableNextColumn();
                LeftPadding(Setting_PaddingWidth, openplanetUIDisplay(dataFileMgr.Ghosts[indexToDisplayMedals[i]].Nickname));
                UI::TableNextRow();
                UI::TableNextColumn();
                counter++;
            }
#endif
#if MP4
            if (indexToDisplayMedals.Length == 0) {LeftPadding(Setting_PaddingWidth, "Map Medals Unavailable.");}
#elif TURBO
            if (indexToDisplayMedals.Length == 0 && (mapSTMIndex <= 0 && mapSTMIndex > 200)) {LeftPadding(Setting_PaddingWidth, "Map Medals Unavailable.");}
#endif
#if MP4 || TURBO
            UI::EndTable();
            UI::Separator();
        }
#endif
    }
    UI::EndChild();
    UI::Separator();

    LeftPaddingDummy(Setting_PaddingWidth);
    if (UI::RedButton(Icons::Times + " Close")) {
        replayListAll = false;
    }
    if (userSelectedReplay) {
        UI::SameLine();
        LeftPaddingDummy(Setting_PaddingWidth);
        if (UI::GreyButton(Icons::Repeat + " Re-enable Autodetect")) {
            selectedIndex = -1;
            selectedDediIndex = -1;
            selectedSTMIndex = -1;
            userSelectedReplay = false;
            autoDetectEnabled = true;
            if (Setting_EnableNotif) {
                UI::ShowNotification("\\$s\\$fff" + Icons::InfoCircle + " Auto-Detect Re-enabled!",
                            "\\$s\\$fffReplays will be automatically detected and updated.",
                            vec4(Setting_NotificationColour.x, Setting_NotificationColour.y, Setting_NotificationColour.z, 0), 5000);
            }
        }
    }
    UI::End();
    UI::PopFont();
}