string formatTime(uint32 time)
{
    uint32 seconds = time / 1000 % 60;
    uint32 minutes = time / 1000 / 60 % 60;
    uint32 hours = time / 1000 / 60 / 60;

    string date = "";

    if (hours > 0) {
        date += formatNumber(hours) + ":";
    }

    date += formatNumber(minutes) + ':' + formatNumber(seconds) + '.' + formatThousandthsNumber(time % 1000);

    return date;
}

string formatSplit(uint32 time)
{
    uint32 seconds = time / 1000 % 60;
    uint32 minutes = time / 1000 / 60 % 60;
    uint32 hours = time / 1000 / 60 / 60;

    string date = "";

    if (hours > 0) {
        date += formatNumber(hours) + ":";
    }

    date += minutes + ':' + formatNumber(seconds) + '.' + formatThousandthsNumber(time % 1000);

    return date;
}

string formatNumber(uint32 number)
{
    if (number < 10) {
        return "0" + number;
    }

    return tostring(number);
}

string formatThousandthsNumber(uint32 number)
{
    if (number < 10) return "00" + number;
    if (number < 100) return "0" + number;

    return tostring(number);
}

string openplanetUIDisplay(const string &in name)
{
    string replacedName = Regex::Replace(name, "\\$([iIoOnNmMwWsSzZtTgG<>]|[lLhHpP](\\[[^\\]]+\\])?)", '');
    return Regex::Replace(replacedName, "\\$", '\\$');
}

string GetManiaplanetColour(vec3 colour) {
    string textColour = "";
    textColour = textColour + (Text::FormatPointer(RGBAColor(colour).r)).SubStr(16, 1);
    textColour = textColour + (Text::FormatPointer(RGBAColor(colour).g)).SubStr(16, 1);
    textColour = textColour + (Text::FormatPointer(RGBAColor(colour).b)).SubStr(16, 1);

    return textColour;
}

bool isCameraTooFar(vec3 player, vec3 camera) {
    float acceptableDistanceX = 10.00;
    float acceptableDistanceY = 10.00;
    
    if (Math::Floor(player.x) >= (Math::Floor(camera.x) + acceptableDistanceX) ||
        Math::Floor(player.x) <= (Math::Floor(camera.x) - acceptableDistanceX)) {return true;}
    if (Math::Floor(player.y) >= (Math::Floor(camera.y) + acceptableDistanceY) ||
        Math::Floor(player.y) <= (Math::Floor(camera.y) - acceptableDistanceY)) {return true;}
    if (Math::Floor(player.z) >= (Math::Floor(camera.z) + acceptableDistanceX) ||
        Math::Floor(player.z) <= (Math::Floor(camera.z) - acceptableDistanceX)) {return true;}
    return false;
}

class CGameGhostStriped {
    string nickname;
    int raceTime;
    int[] checkpoints;

    CGameGhostStriped() {
        nickname = '';
        raceTime = -1;
        checkpoints = {};
    }
}

bool TimeExists(CGameGhostStriped[]@ ghosts, int raceTime) {
    for (uint i = 0; i < ghosts.Length; i++) if (ghosts[i].raceTime == raceTime) return true;
    return false;
}

CGameGhostStriped@ CreateGhost(const string &in nickname, int time, MwFastBuffer<int> checkpoints) {
    CGameGhostStriped@ ret = CGameGhostStriped();
    ret.nickname = nickname;
    ret.raceTime = time;
    int[] cps = {};
    for (uint i = 0; i < checkpoints.Length; i++) {cps.InsertLast(checkpoints[i]);}
    ret.checkpoints = cps;
    return ret;
}

CGameGhostStriped@ CreateGhost(const string &in nickname, int time, int[] checkpoints) {
    CGameGhostStriped@ ret = CGameGhostStriped();
    ret.nickname = nickname;
    ret.raceTime = time;
    ret.checkpoints = checkpoints;
    return ret;
}

CGameGhostStriped@ CreateGhost(CGameGhostScript@ ghost) {
    CGameGhostStriped@ ret = CGameGhostStriped();
#if TMNEXT
    ret.nickname = ghost.Nickname.Contains('Personal best') ? GetPlayerName() : ghost.Nickname;
    auto result = ghost.Result;
#elif MP4
    ret.nickname = ghost.Nickname;
    auto result = ghost.Result;
#elif TURBO
    ret.nickname = ghost.Nickname;
    auto result = ghost.RaceResult;
#endif
    ret.raceTime = result.Time;
    int[] cps = {};
    for (uint i = 0; i < result.Checkpoints.Length; i++) {cps.InsertLast(result.Checkpoints[i]);}
    ret.checkpoints = cps;
    return ret;
}

CGameGhostStriped@ CreateGhost(int dedi) {
    CGameGhostStriped@ ret = CGameGhostStriped();
    if (dedi != 2) ret.nickname = GetPlayerName();
    else ret.nickname = dedimaniaLogins[2];
    ret.raceTime = dediReplayCPs[dedi][dediReplayCPs[dedi].Length - 1];
    ret.checkpoints = dediReplayCPs[dedi];
    return ret;
}

#if TURBO
CGameGhostStriped@ CreateGhost(STMRecord@ stm) {
    CGameGhostStriped@ ret = CGameGhostStriped();
    ret.nickname = stm.playerName;
    ret.raceTime = stm.raceTime;
    ret.checkpoints = stm.checkpointTimes;
    return ret;
}
#endif

uint GetCPValue(bool getFinishTime = false) {
#if TMNEXT
    return player.LastCpTime;
#elif MP4 || TURBO
    return GetPlayer() is null ? MAX_INT : GetPlayer().CurCheckpointRaceTime;
#endif
}


int GetLapValue() {
#if MP4 || TURBO
    return GetPlayer() is null ? 0 : GetPlayer().CurrentNbLaps;
#elif TMNEXT
    if (!GetRootMap().TMObjective_IsLapRace) return 0;
    if (currCheckpointTimes.Length < mapCheckpointCount) return 0;
    else return int(Math::Floor(currCheckpointTimes.Length / mapCheckpointCount));
#endif
}

vec3 GetPosition() {
#if TMNEXT
    auto player = VehicleState::GetViewingPlayer();
    if (player is null) return vec3(0, 0, 0);
    auto vis = VehicleState::GetVis(GetApp().GameScene, player).AsyncState;
    if (vis is null) return vec3(0, 0, 0);
    return vis.Position;
#elif MP4 || TURBO
    return GetPlayer() is null ? vec3(0, 0, 0) : GetPlayer().Position;
#endif
}

// #if TMNEXT
// string GetRoundPosition() {
//     if (IsModeRounds()) return GeneratePositionText(player.RaceRank);
//     else return GeneratePositionText(player.RaceRespawnRank); 
// }
// #endif

CGameCtnChallenge@ GetRootMap() {
#if MP4 || TMNEXT
    return GetApp().RootMap;
#elif TURBO
    return GetApp().Challenge;
#endif
}

#if TMNEXT
CSmScriptPlayer@ GetScriptAPI() {return cast<CSmScriptPlayer>(GetPlayer().ScriptAPI);}

SGamePlaygroundUIConfig::EUISequence CurrentUISequence() {
    return GetApp().CurrentPlayground.GameTerminals[0].UISequence_Current;
}
#endif

bool IsSpawned() {
#if MP4 || TURBO
    return GetPlayer() is null ? false : GetPlayer().IsSpawned;
#elif TMNEXT
    return player.IsSpawned;
#endif
}

int GetRaceState() {
#if MP4 || TURBO
    return GetPlayer() is null ? 0 : GetPlayer().RaceState;
#elif TMNEXT
    if (GetPlayer() is null) return 0;
    CSmScriptPlayer@ scriptAPI = cast<CSmScriptPlayer@>(GetPlayer().ScriptAPI);
    if (scriptAPI.Post == 0) return 0;
    else if (CurrentUISequence() == CGamePlaygroundUIConfig::EUISequence::Finish) return 2;
    else if (scriptAPI.Post == 2) return 1;
    return -1;
#endif
}

uint GetCPTime(bool getFinishTime = false) {return GetCPValue(getFinishTime);}

#if MP4 || TURBO
uint GetCPLapTime() {return GetPlayer() is null ? MAX_INT : GetPlayer().CurCheckpointLapTime;}
#elif TMNEXT

#endif


MwFastBuffer<int> GetPlayerCPs() {
#if MP4
    return GetPlayer().ScriptAPI.CurRace.Checkpoints;
#elif TURBO
    return GetPlayer().CurRace.Checkpoints;
#elif TMNEXT
    int[] cps = player.CpTimes;
    cps.RemoveAt(0);
    return ToBuffer(cps);
#endif
}

MwFastBuffer<int> ToBuffer(int[] arr) {
    MwFastBuffer<int> buffer = MwFastBuffer<int>();
    for (uint i = 0; i < arr.Length; i++) buffer.Add(arr[i]);
    return buffer;
}

#if TMNEXT
    CGameDataFileManagerScript@ GetDataMgr() {return GetApp().PlaygroundScript !is null ? GetApp().PlaygroundScript.DataFileMgr : GetApp().Network.ClientManiaAppPlayground.DataFileMgr;}
    wstring GetPlayerName() {return GetApp().LocalPlayerInfo.Name;}
#elif MP4
    CGameDataFileManagerScript@ GetDataMgr() {return GetApp().PlaygroundScript !is null ? GetApp().PlaygroundScript.DataFileMgr : null;}
    string GetPlayerName() {return GetApp().CurrentProfile.AccountSettings.NickName;}
#elif TURBO
    CGameDataManagerScript@ GetDataMgr() {return GetApp().PlaygroundScript !is null ? GetApp().PlaygroundScript.DataMgr : null;;}
    string GetPlayerName() {return GetPlayer().Name;}
#endif

bool IsMapMultilap() {return GetRootMap().TMObjective_IsLapRace && GetRootMap().TMObjective_NbLaps > 1;}

vec3 GetRoll() {
    auto m_visible = Dev::GetOffsetNod((Dev::GetOffsetNod(VehicleState::GetViewingPlayer(), 0x14)), 0x84);
    int base = 0xC4;
    return vec3(Dev::GetOffsetFloat(m_visible, base - 36), Dev::GetOffsetFloat(m_visible, base - 24), Dev::GetOffsetFloat(m_visible, base - 12));
}

vec3 GetPitch() {
    auto m_visible = Dev::GetOffsetNod((Dev::GetOffsetNod(VehicleState::GetViewingPlayer(), 0x14)), 0x84);
    int base = 0xC4;
    return vec3(Dev::GetOffsetFloat(m_visible, base - 32), Dev::GetOffsetFloat(m_visible, base - 20), Dev::GetOffsetFloat(m_visible, base - 8));
}

vec3 GetYaw() {
    auto m_visible = Dev::GetOffsetNod((Dev::GetOffsetNod(VehicleState::GetViewingPlayer(), 0x14)), 0x84);
    int base = 0xC4;
    return vec3(Dev::GetOffsetFloat(m_visible, base - 28), Dev::GetOffsetFloat(m_visible, base - 16), Dev::GetOffsetFloat(m_visible, base - 4));
}